@@ -1,6 +1,4 @@
 /*
- * $Id: xftfreetype.c,v 1.5 2005/07/03 07:00:57 daniels Exp $
- *
  * Copyright © 2000 Keith Packard
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -24,7 +22,7 @@
 
 #include "xftint.h"
 
-FT_Library  _XftFTlibrary;
+_X_HIDDEN FT_Library  _XftFTlibrary;
 
 #define FT_Matrix_Equal(a,b)	((a)->xx == (b)->xx && \
 				 (a)->yy == (b)->yy && \
@@ -35,7 +33,7 @@ FT_Library  _XftFTlibrary;
  */
 
 static XftFtFile *_XftFtFiles;
-int XftMaxFreeTypeFiles = 5;
+static int XftMaxFreeTypeFiles = 5;
 
 static XftFtFile *
 _XftGetFile (const FcChar8 *file, int id)
@@ -43,7 +41,7 @@ _XftGetFile (const FcChar8 *file, int id)
     XftFtFile	*f;
 
     if (!XftInitFtLibrary ())
-	return 0;
+	return NULL;
 
     for (f = _XftFtFiles; f; f = f->next)
     {
@@ -58,23 +56,23 @@ _XftGetFile (const FcChar8 *file, int id)
     }
     f = malloc (sizeof (XftFtFile) + strlen ((char *) file) + 1);
     if (!f)
-	return 0;
-    
+	return NULL;
+
     XftMemAlloc (XFT_MEM_FILE, sizeof (XftFtFile) + strlen ((char *) file) + 1);
     if (XftDebug () & XFT_DBG_REF)
     	printf ("FontFile %s/%d matches new\n",
 		file, id);
     f->next = _XftFtFiles;
     _XftFtFiles = f;
-    
+
     f->ref = 1;
-    
+
     f->file = (char *) (f+1);
     strcpy (f->file, (char *) file);
     f->id = id;
-    
+
     f->lock = 0;
-    f->face = 0;
+    f->face = NULL;
     f->xsize = 0;
     f->ysize = 0;
     f->matrix.xx = f->matrix.xy = f->matrix.yx = f->matrix.yy = 0;
@@ -88,13 +86,13 @@ _XftGetFaceFile (FT_Face face)
 
     f = malloc (sizeof (XftFtFile));
     if (!f)
-	return 0;
+	return NULL;
     XftMemAlloc (XFT_MEM_FILE, sizeof(XftFtFile));
-    f->next = 0;
-    
+    f->next = NULL;
+
     f->ref = 1;
-    
-    f->file = 0;
+
+    f->file = NULL;
     f->id = 0;
     f->lock = 0;
     f->face = face;
@@ -141,7 +139,7 @@ _XftUncacheFiles (void)
 		printf ("Discard file %s/%d from cache\n",
 			f->file, f->id);
 	    FT_Done_Face (f->face);
-	    f->face = 0;
+	    f->face = NULL;
 	}
     }
 }
@@ -156,7 +154,7 @@ _XftLockFile (XftFtFile *f)
 	    printf ("Loading file %s/%d\n", f->file, f->id);
 	if (FT_New_Face (_XftFTlibrary, f->file, f->id, &f->face))
 	    --f->lock;
-	    
+
 	f->xsize = 0;
 	f->ysize = 0;
 	f->matrix.xx = f->matrix.xy = f->matrix.yx = f->matrix.yy = 0;
@@ -166,7 +164,7 @@ _XftLockFile (XftFtFile *f)
 }
 
 static void
-_XftLockError (char *reason)
+_XftLockError (const char *reason)
 {
     fprintf (stderr, "Xft: locking error %s\n", reason);
 }
@@ -178,23 +176,18 @@ _XftUnlockFile (XftFtFile *f)
 	_XftLockError ("too many file unlocks");
 }
 
-#if HAVE_FT_BITMAP_SIZE_Y_PPEM
 #define X_SIZE(face,i) ((face)->available_sizes[i].x_ppem)
 #define Y_SIZE(face,i) ((face)->available_sizes[i].y_ppem)
-#else
-#define X_SIZE(face,i) ((face)->available_sizes[i].width << 6)
-#define Y_SIZE(face,i) ((face)->available_sizes[i].height << 6)
-#endif
 
-FcBool
+_X_HIDDEN FcBool
 _XftSetFace (XftFtFile *f, FT_F26Dot6 xsize, FT_F26Dot6 ysize, FT_Matrix *matrix)
 {
     FT_Face face = f->face;
-    
+
     if (f->xsize != xsize || f->ysize != ysize)
     {
 	if (XftDebug() & XFT_DBG_GLYPH)
-	    printf ("Set face size to %dx%d (%dx%d)\n", 
+	    printf ("Set face size to %dx%d (%dx%d)\n",
 		    (int) (xsize >> 6), (int) (ysize >> 6), (int) xsize, (int) ysize);
 	/*
 	 * Bitmap only faces must match exactly, so find the closest
@@ -219,19 +212,16 @@ _XftSetFace (XftFtFile *f, FT_F26Dot6 xsize, FT_F26Dot6 ysize, FT_Matrix *matrix
 		    best = i;
 		}
 	    }
-	    /* 
+	    /*
 	     * Freetype 2.1.7 and earlier used width/height
 	     * for matching sizes in the BDF and PCF loaders.
 	     * This has been fixed for 2.1.8.  Because BDF and PCF
 	     * files have but a single strike per file, we can
 	     * simply try both sizes.
 	     */
-	    if (
-#if HAVE_FT_BITMAP_SIZE_Y_PPEM
-		FT_Set_Char_Size (face, face->available_sizes[best].x_ppem,
+	    if (FT_Set_Char_Size (face, face->available_sizes[best].x_ppem,
 				  face->available_sizes[best].y_ppem, 0, 0) != 0
 		&&
-#endif
 		FT_Set_Char_Size (face, face->available_sizes[best].width << 6,
 				  face->available_sizes[best].height << 6,
 				  0, 0) != 0)
@@ -257,7 +247,7 @@ _XftSetFace (XftFtFile *f, FT_F26Dot6 xsize, FT_F26Dot6 ysize, FT_Matrix *matrix
 		    (double) matrix->xy / 0x10000,
 		    (double) matrix->yx / 0x10000,
 		    (double) matrix->yy / 0x10000);
-	FT_Set_Transform (face, matrix, 0);
+	FT_Set_Transform (face, matrix, NULL);
 	f->matrix = *matrix;
     }
     return True;
@@ -267,7 +257,7 @@ static void
 _XftReleaseFile (XftFtFile *f)
 {
     XftFtFile	**prev;
-    
+
     if (--f->ref != 0)
         return;
     if (f->lock)
@@ -285,7 +275,7 @@ _XftReleaseFile (XftFtFile *f)
 	if (f->face)
 	    FT_Done_Face (f->face);
     }
-    XftMemFree (XFT_MEM_FILE, 
+    XftMemFree (XFT_MEM_FILE,
 		sizeof (XftFtFile) + (f->file ? strlen (f->file) + 1 : 0));
     free (f);
 }
@@ -345,13 +335,13 @@ _XftHashSize (FcChar32 num_unicode)
     return hash;
 }
 
-FT_Face
+_X_EXPORT FT_Face
 XftLockFace (XftFont *public)
 {
     XftFontInt	*font = (XftFontInt *) public;
     XftFontInfo	*fi = &font->info;
     FT_Face	face;
-    
+
     face = _XftLockFile (fi->file);
     /*
      * Make sure the face is usable at the requested size
@@ -359,12 +349,12 @@ XftLockFace (XftFont *public)
     if (face && !_XftSetFace (fi->file, fi->xsize, fi->ysize, &fi->matrix))
     {
 	_XftUnlockFile (fi->file);
-	face = 0;
+	face = NULL;
     }
     return face;
 }
 
-void
+_X_EXPORT void
 XftUnlockFace (XftFont *public)
 {
     XftFontInt	*font = (XftFontInt *) public;
@@ -381,29 +371,35 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
     double	    aspect;
     FcMatrix	    *font_matrix;
     FcBool	    hinting, vertical_layout, autohint, global_advance;
-#ifdef FC_HINT_STYLE
     int             hint_style;
-#endif
     FcChar32	    hash, *hashp;
     FT_Face	    face;
     int		    nhash;
+    FcBool	    bitmap;
 
     if (!info)
 	return FcFalse;
 
+    /*
+     * Initialize the whole XftFontInfo so that padding doesn't interfere with
+     * hash or XftFontInfoEqual().
+     */
+
+    memset (fi, '\0', sizeof(*fi));
+
     /*
      * Find the associated file
      */
     switch (FcPatternGetString (pattern, FC_FILE, 0, &filename)) {
     case FcResultNoMatch:
-	filename = 0;
+	filename = NULL;
 	break;
     case FcResultMatch:
 	break;
     default:
 	goto bail0;
     }
-    
+
     switch (FcPatternGetInteger (pattern, FC_INDEX, 0, &id)) {
     case FcResultNoMatch:
 	id = 0;
@@ -413,14 +409,12 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
     default:
 	goto bail0;
     }
-    
+
     if (filename)
 	fi->file = _XftGetFile (filename, id);
     else if (FcPatternGetFTFace (pattern, FC_FT_FACE, 0, &face) == FcResultMatch
 	     && face)
 	fi->file = _XftGetFaceFile (face);
-    else
-	fi->file = 0;
     if (!fi->file)
         goto bail0;
 
@@ -432,7 +426,7 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
 
     if (FcPatternGetDouble (pattern, FC_ASPECT, 0, &aspect) != FcResultMatch)
 	aspect = 1.0;
-    
+
     fi->ysize = (FT_F26Dot6) (dsize * 64.0);
     fi->xsize = (FT_F26Dot6) (dsize * aspect * 64.0);
 
@@ -451,7 +445,7 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
     default:
 	goto bail1;
     }
-    
+
     /*
      * Get rgba value
      */
@@ -464,7 +458,20 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
     default:
 	goto bail1;
     }
-    
+
+    /*
+     * Get lcd_filter value
+     */
+    switch (FcPatternGetInteger (pattern, FC_LCD_FILTER, 0, &fi->lcd_filter)) {
+    case FcResultNoMatch:
+	fi->lcd_filter = FC_LCD_DEFAULT;
+	break;
+    case FcResultMatch:
+	break;
+    default:
+	goto bail1;
+    }
+
     /*
      * Get matrix and transform values
      */
@@ -485,8 +492,8 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
 
     fi->transform = (fi->matrix.xx != 0x10000 || fi->matrix.xy != 0 ||
 		     fi->matrix.yx != 0 || fi->matrix.yy != 0x10000);
-    
-    /* 
+
+    /*
      * Get render value, set to false if no Render extension present
      */
     if (info->hasRender)
@@ -503,16 +510,30 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
     }
     else
 	fi->render = FcFalse;
-    
+
     /*
      * Compute glyph load flags
      */
     fi->load_flags = FT_LOAD_DEFAULT;
 
+#ifndef XFT_EMBEDDED_BITMAP
+#define XFT_EMBEDDED_BITMAP "embeddedbitmap"
+#endif
+
+    switch (FcPatternGetBool (pattern, XFT_EMBEDDED_BITMAP, 0, &bitmap)) {
+    case FcResultNoMatch:
+	bitmap = FcFalse;
+	break;
+    case FcResultMatch:
+	break;
+    default:
+	goto bail1;
+    }
+
     /* disable bitmaps when anti-aliasing or transforming glyphs */
-    if (fi->antialias || fi->transform)
+    if ((!bitmap && fi->antialias) || fi->transform)
 	fi->load_flags |= FT_LOAD_NO_BITMAP;
-    
+
     /* disable hinting if requested */
     switch (FcPatternGetBool (pattern, FC_HINTING, 0, &hinting)) {
     case FcResultNoMatch:
@@ -524,7 +545,6 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
 	goto bail1;
     }
 
-#ifdef FC_EMBOLDEN
     switch (FcPatternGetBool (pattern, FC_EMBOLDEN, 0, &fi->embolden)) {
     case FcResultNoMatch:
 	fi->embolden = FcFalse;
@@ -534,11 +554,7 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
     default:
 	goto bail1;
     }
-#else
-    fi->embolden = FcFalse;
-#endif
-    
-#ifdef FC_HINT_STYLE
+
     switch (FcPatternGetInteger (pattern, FC_HINT_STYLE, 0, &hint_style)) {
     case FcResultNoMatch:
 	hint_style = FC_HINT_FULL;
@@ -548,12 +564,9 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
     default:
 	goto bail1;
     }
-#endif
 
     if (!hinting
-#ifdef FC_HINT_STYLE
 	|| hint_style == FC_HINT_NONE
-#endif
 	)
     {
 	fi->load_flags |= FT_LOAD_NO_HINTING;
@@ -564,15 +577,11 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
      */
     if (fi->antialias)
     {
-#ifdef FC_HINT_STYLE
-#ifdef FT_LOAD_TARGET_LIGHT
 	if (FC_HINT_NONE < hint_style && hint_style < FC_HINT_FULL)
 	{
 	    fi->load_flags |= FT_LOAD_TARGET_LIGHT;
 	}
 	else
-#endif
-#endif
 	{
 	    /* autohinter will snap stems to integer widths, when
 	     * the LCD targets are used.
@@ -580,24 +589,18 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
 	    switch (fi->rgba) {
 	    case FC_RGBA_RGB:
 	    case FC_RGBA_BGR:
-#ifdef FT_LOAD_TARGET_LCD
 		fi->load_flags |= FT_LOAD_TARGET_LCD;
-#endif
 		break;
 	    case FC_RGBA_VRGB:
 	    case FC_RGBA_VBGR:
-#ifdef FT_LOAD_TARGET_LCD_V
 		fi->load_flags |= FT_LOAD_TARGET_LCD_V;
-#endif
 		break;
 	    }
 	}
     }
-#ifdef FT_LOAD_TARGET_MONO
     else
 	fi->load_flags |= FT_LOAD_TARGET_MONO;
-#endif
-    
+
     /* set vertical layout if requested */
     switch (FcPatternGetBool (pattern, FC_VERTICAL_LAYOUT, 0, &vertical_layout)) {
     case FcResultNoMatch:
@@ -639,7 +642,7 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
 
     if (!global_advance)
 	fi->load_flags |= FT_LOAD_IGNORE_GLOBAL_ADVANCE_WIDTH;
-    
+
     /*
      * Get requested spacing value
      */
@@ -652,7 +655,7 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
     default:
 	goto bail1;
     }
-    
+
     /*
      * Check for minspace
      */
@@ -667,7 +670,7 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
 	goto bail1;
     }
     /*
-     * Check for fixed pixel spacing 
+     * Check for fixed pixel spacing
      */
     switch (FcPatternGetInteger (pattern, FC_CHAR_WIDTH, 0, &fi->char_width)) {
     case FcResultNoMatch:
@@ -691,15 +694,15 @@ XftFontInfoFill (Display *dpy, _Xconst FcPattern *pattern, XftFontInfo *fi)
     while (nhash--)
 	hash += *hashp++;
     fi->hash = hash;
-    
+
     /*
      * All done
      */
     return FcTrue;
-    
+
 bail1:
     _XftReleaseFile (fi->file);
-    fi->file = 0;
+    fi->file = NULL;
 bail0:
     return FcFalse;
 }
@@ -717,18 +720,18 @@ XftFontInfoCreate (Display *dpy, _Xconst FcPattern *pattern)
     XftFontInfo	*fi = malloc (sizeof (XftFontInfo));
 
     if (!fi)
-	return 0;
-    
+	return NULL;
+
     if (!XftFontInfoFill (dpy, pattern, fi))
     {
 	free (fi);
-	fi = 0;
+	fi = NULL;
     }
     XftMemAlloc (XFT_MEM_FONT, sizeof (XftFontInfo));
     return fi;
 }
 
-void
+_X_EXPORT void
 XftFontInfoDestroy (Display *dpy, XftFontInfo *fi)
 {
     XftFontInfoEmpty (dpy, fi);
@@ -736,21 +739,21 @@ XftFontInfoDestroy (Display *dpy, XftFontInfo *fi)
     free (fi);
 }
 
-FcChar32
+_X_EXPORT FcChar32
 XftFontInfoHash (_Xconst XftFontInfo *fi)
 {
     return fi->hash;
 }
-    
-FcBool
+
+_X_EXPORT FcBool
 XftFontInfoEqual (_Xconst XftFontInfo *a, _Xconst XftFontInfo *b)
 {
     return memcmp ((void *) a, (void *) b, sizeof (XftFontInfo)) == 0;
 }
 
-XftFont *
-XftFontOpenInfo (Display	*dpy, 
-		 FcPattern	*pattern, 
+_X_EXPORT XftFont *
+XftFontOpenInfo (Display	*dpy,
+		 FcPattern	*pattern,
 		 XftFontInfo	*fi)
 {
     XftDisplayInfo	*info = _XftDisplayInfoGet (dpy, True);
@@ -770,7 +773,7 @@ XftFontOpenInfo (Display	*dpy,
     int			num_glyphs;
 
     if (!info)
-	return 0;
+	return NULL;
     /*
      * Find a matching previously opened font
      */
@@ -785,14 +788,14 @@ XftFontOpenInfo (Display	*dpy,
 	}
 
     /*
-     * No existing font, create another.  
+     * No existing font, create another.
      */
-    
+
     if (XftDebug () & XFT_DBG_CACHE)
 	printf ("New font %s/%d size %dx%d\n",
 		fi->file->file, fi->file->id,
 		(int) fi->xsize >> 6, (int) fi->ysize >> 6);
-		
+
     if (FcPatternGetInteger (pattern, XFT_MAX_GLYPH_MEMORY, 0,
 			     &max_glyph_memory) != FcResultMatch)
 	max_glyph_memory = XFT_FONT_MAX_GLYPH_MEMORY;
@@ -813,8 +816,8 @@ XftFontOpenInfo (Display	*dpy,
     if (FcPatternGetCharSet (pattern, FC_CHARSET, 0, &charset) == FcResultMatch)
 	charset = FcCharSetCopy (charset);
     else
-	charset = FcFreeTypeCharSet (face, FcConfigGetBlanks (0));
-    
+	charset = FcFreeTypeCharSet (face, FcConfigGetBlanks (NULL));
+
     antialias = fi->antialias;
     if (!(face->face_flags & FT_FACE_FLAG_SCALABLE))
 	antialias = FcFalse;
@@ -842,13 +845,13 @@ XftFontOpenInfo (Display	*dpy,
 	{
 	    format = XRenderFindStandardFormat (dpy, PictStandardA1);
 	}
-	
+
 	if (!format)
 	    goto bail2;
     }
     else
-	format = 0;
-    
+	format = NULL;
+
     if (charset)
     {
 	num_unicode = FcCharSetCount (charset);
@@ -861,17 +864,17 @@ XftFontOpenInfo (Display	*dpy,
 	hash_value = 0;
 	rehash_value = 0;
     }
-    
+
     /*
      * Sometimes the glyphs are numbered 1..n, other times 0..n-1,
      * accept either numbering scheme by making room in the table
      */
     num_glyphs = face->num_glyphs + 1;
-    alloc_size = (sizeof (XftFontInt) + 
+    alloc_size = (sizeof (XftFontInt) +
 		  num_glyphs * sizeof (XftGlyph *) +
 		  hash_value * sizeof (XftUcsHash));
     font = malloc (alloc_size);
-    
+
     if (!font)
 	goto bail2;
 
@@ -883,12 +886,12 @@ XftFontOpenInfo (Display	*dpy,
     if (fi->transform)
     {
 	FT_Vector	vector;
-	
+
 	vector.x = 0;
 	vector.y = face->size->metrics.descender;
 	FT_Vector_Transform (&vector, &fi->matrix);
 	descent = -(vector.y >> 6);
-	
+
 	vector.x = 0;
 	vector.y = face->size->metrics.ascender;
 	FT_Vector_Transform (&vector, &fi->matrix);
@@ -916,7 +919,7 @@ XftFontOpenInfo (Display	*dpy,
     font->public.ascent = ascent;
     font->public.descent = descent;
     font->public.height = height;
-    
+
     if (fi->char_width)
 	font->public.max_advance_width = fi->char_width;
     else
@@ -934,7 +937,7 @@ XftFontOpenInfo (Display	*dpy,
     }
     font->public.charset = charset;
     font->public.pattern = pattern;
-    
+
     /*
      * Management fields
      */
@@ -942,10 +945,10 @@ XftFontOpenInfo (Display	*dpy,
 
     font->next = info->fonts;
     info->fonts = &font->public;
-    
+
     font->hash_next = *bucket;
     *bucket = &font->public;
-    
+
     /*
      * Copy the info over
      */
@@ -960,7 +963,7 @@ XftFontOpenInfo (Display	*dpy,
      * bump XftFile reference count
      */
     font->info.file->ref++;
-    
+
     /*
      * Per glyph information
      */
@@ -983,41 +986,41 @@ XftFontOpenInfo (Display	*dpy,
      */
     font->glyphset = 0;
     font->format = format;
-    
+
     /*
      * Glyph memory management fields
      */
     font->glyph_memory = 0;
     font->max_glyph_memory = max_glyph_memory;
     font->use_free_glyphs = info->use_free_glyphs;
-    
+
     _XftUnlockFile (fi->file);
 
     return &font->public;
-    
+
 bail2:
     FcCharSetDestroy (charset);
 bail1:
     _XftUnlockFile (fi->file);
 bail0:
-    return 0;
+    return NULL;
 }
 
-XftFont *
+_X_EXPORT XftFont *
 XftFontOpenPattern (Display *dpy, FcPattern *pattern)
 {
     XftFontInfo	    info;
     XftFont	    *font;
 
     if (!XftFontInfoFill (dpy, pattern, &info))
-	return 0;
+	return NULL;
 
     font = XftFontOpenInfo (dpy, pattern, &info);
     XftFontInfoEmpty (dpy, &info);
     return font;
 }
 
-XftFont *
+_X_EXPORT XftFont *
 XftFontCopy (Display *dpy, XftFont *public)
 {
     XftFontInt	    *font = (XftFontInt *) public;
@@ -1032,7 +1035,7 @@ XftFontDestroy (Display *dpy, XftFont *public)
     XftDisplayInfo  *info = _XftDisplayInfoGet (dpy, False);
     XftFontInt	    *font = (XftFontInt *) public;
     int		    i;
-    
+
     /* note reduction in memory use */
     if (info)
 	info->glyph_memory -= font->glyph_memory;
@@ -1052,11 +1055,11 @@ XftFontDestroy (Display *dpy, XftFont *public)
 	    free (xftg);
 	}
     }
-    
+
     /* Free the pattern and the charset */
     FcPatternDestroy (font->public.pattern);
     FcCharSetDestroy (font->public.charset);
-    
+
     /* Finally, free the font structure */
     XftMemFree (XFT_MEM_FONT, sizeof (XftFontInt) +
 		font->num_glyphs * sizeof (XftGlyph *) +
@@ -1069,7 +1072,7 @@ XftFontFindNthUnref (XftDisplayInfo *info, int n)
 {
     XftFont	*public;
     XftFontInt	*font;
-    
+
     for (public = info->fonts; public; public = font->next)
     {
 	font = (XftFontInt*) public;
@@ -1079,7 +1082,7 @@ XftFontFindNthUnref (XftDisplayInfo *info, int n)
     return public;
 }
 
-void
+_X_HIDDEN void
 XftFontManageMemory (Display *dpy)
 {
     XftDisplayInfo  *info = _XftDisplayInfoGet (dpy, False);
@@ -1125,15 +1128,15 @@ XftFontManageMemory (Display *dpy)
     }
 }
 
-void
+_X_EXPORT void
 XftFontClose (Display *dpy, XftFont *public)
 {
     XftDisplayInfo  *info = _XftDisplayInfoGet (dpy, False);
     XftFontInt	    *font = (XftFontInt *) public;
-    
+
     if (--font->ref != 0)
 	return;
-    
+
     if (info)
     {
 	++info->num_unref_fonts;
@@ -1145,7 +1148,7 @@ XftFontClose (Display *dpy, XftFont *public)
     }
 }
 
-FcBool
+_X_EXPORT FcBool
 XftInitFtLibrary (void)
 {
     if (_XftFTlibrary)
